@@ -14,7 +14,11 @@ export STEAMSCRIPT
 bootstrapscript="$(readlink -f "$STEAMSCRIPT")"
 bootstrapdir="$(dirname "$bootstrapscript")"
 
-export STEAMSCRIPT_VERSION=1.0.0.70
+log () {
+    echo "bin_steam.sh[$$]: $*" >&2 || :
+}
+
+export STEAMSCRIPT_VERSION=1.0.0.74
 
 # Set up domain for script localization
 export TEXTDOMAIN=steam
@@ -81,7 +85,7 @@ function setup_variables()
 			DEFAULTSTEAMDIR="$STEAM_DATA_HOME/SteamBeta"
 			;;
 		*)
-			echo $"Unknown Steam package '$STEAMPACKAGE'"
+			log $"Unknown Steam package '$STEAMPACKAGE'"
 			exit 1
 			;;
 	esac
@@ -100,11 +104,11 @@ function install_bootstrap()
 	omask="$(umask)"
 	umask 0077
 
-	echo $"Setting up Steam content in $STEAMDIR"
+	log $"Setting up Steam content in $STEAMDIR"
 	mkdir -p "$STEAMDIR"
 	cd "$STEAMDIR"
 	if ! tar xJf "$LAUNCHSTEAMBOOTSTRAPFILE" ; then
-		echo $"Failed to extract $LAUNCHSTEAMBOOTSTRAPFILE, aborting installation."
+		log $"Failed to extract $LAUNCHSTEAMBOOTSTRAPFILE, aborting installation."
 		exit 1
 	fi
 	ln -fns "$STEAMDIR" "$STEAMDATALINK"
@@ -117,7 +121,7 @@ function install_bootstrap()
 	test -f "${XDG_CONFIG_HOME:-$HOME/.config}/user-dirs.dirs" && source "${XDG_CONFIG_HOME:-$HOME/.config}/user-dirs.dirs"
 	DESKTOP_DIR="${XDG_DESKTOP_DIR:-$HOME/Desktop}"
 
-	if [ -d "$DESKTOP_DIR" ] && [ "x$bootstrapdir" = "x/usr/lib/$STEAMPACKAGE" ]; then
+	if [ -d "$DESKTOP_DIR" ] && [ "$bootstrapdir" = "/usr/lib/$STEAMPACKAGE" ]; then
 		# There might be a symlink in place already, in such case we do nothing
 		if [ ! -L "$DESKTOP_DIR/$STEAMPACKAGE.desktop" ]; then
 			cp "$bootstrapdir/$STEAMPACKAGE.desktop" "$DESKTOP_DIR"
@@ -168,10 +172,10 @@ setup_variables
 if ! check_bootstrap "$LAUNCHSTEAMDIR"; then
 	# See if we just need to recreate the data link
 	if check_bootstrap "$DEFAULTSTEAMDIR"; then
-		echo $"Repairing installation, linking $STEAMDATALINK to $DEFAULTSTEAMDIR"
+		log $"Repairing installation, linking $STEAMDATALINK to $DEFAULTSTEAMDIR"
 		repair_bootstrap "$DEFAULTSTEAMDIR"
 	elif check_bootstrap "$CLASSICSTEAMDIR"; then
-		echo $"Repairing installation, linking $STEAMDATALINK to $CLASSICSTEAMDIR"
+		log $"Repairing installation, linking $STEAMDATALINK to $CLASSICSTEAMDIR"
 		repair_bootstrap "$CLASSICSTEAMDIR"
 	fi
 fi
