@@ -8,13 +8,21 @@
 """
 
 import argparse
+import glob
+import logging
 import os
 import re
 import shlex
-import stat
 import subprocess
 import sys
-import tempfile
+
+from steam_launcher.launcherutils import run_subprocess
+
+try:
+    from shlex import quote
+except ImportError:
+    # Python 3.2 (not really supported but let's not deliberately break it)
+    from pipes import quote
 
 try:
     import typing
@@ -23,18 +31,31 @@ except ImportError:
 
 try:
     import apt
+    from aptsources.sourceslist import SourceEntry
 except ImportError:
     sys.stderr.write("Couldn't import apt, please install python3-apt or "
                      "update steamdeps for your distribution.\n")
     sys.exit(3)
 
+
+logger = logging.getLogger('steamdeps')
+
 # This is the set of supported Steam runtime environments
 SUPPORTED_STEAM_RUNTIME = ['1']
 
 # This is the set of supported dependency formats
 SUPPORTED_STEAM_DEPENDENCY_VERSION = ['1']
 
+# Environment variables that need to be passed through when we re-exec
+# under pkexec
+PASS_THROUGH_ENV_VARS = (
+    'SL_TEST_NVIDIA_VERSION',
+    'STEAM_LAUNCHER_VERBOSE',
+    'XDG_CURRENT_DESKTOP',
+)
+
 _arch = None
+_foreign_architectures = None
 
 
 class OsRelease:
@@ -112,15 +133,18 @@ def get_arch():
     return _arch
 
 
-###
-def get_full_package_name(name):
+def get_foreign_architectures():
     """
-    Get the full name of a package, qualified by architecture
+    Get the enabled foreign architectures
     """
-    if name.find(":") < 0:
-        return name + ":" + get_arch()
-    else:
-        return name
+    global _foreign_architectures
+
+    if _foreign_architectures is None:
+        _foreign_architectures = subprocess.check_output(
+            ['dpkg', '--print-foreign-architectures'],
+            universal_newlines=True,
+        ).splitlines()
+    return _foreign_architectures
 
 
 # N.B. Version checks are not supported on virtual packages
@@ -221,6 +245,72 @@ def choose_xdg_portal_backend(default_backend="xdg-desktop-portal-gtk"):
         return None
 
 
+def expected_nvidia_packages():
+    # type: (...) -> typing.Dict[str, Package]
+    # Returns a map like:
+    # {"nvidia-driver-libs:amd64": Package("nvidia-driver-libs:amd64", [])}
+    nvidia_packages_to_check = {}       # type: typing.Dict[str, Package]
+    nvidia_packages_to_expect = {}      # type: typing.Dict[str, Package]
+
+    if get_arch() != 'amd64':
+        return nvidia_packages_to_expect
+
+    try:
+        nvidia_version = os.environ.get("SL_TEST_NVIDIA_VERSION")
+        if not nvidia_version:
+            with open('/sys/module/nvidia/version') as f:
+                nvidia_version = f.read()
+        nvidia_major_version = nvidia_version.split('.')[0]
+    except OSError:
+        # If the Nvidia module is not loaded, it's safe to assume that the
+        # Nvidia drivers are not in use. No need to install anything else.
+        return nvidia_packages_to_expect
+
+    for name in (
+        # Debian packages
+        "nvidia-driver-libs:amd64",
+        "nvidia-legacy-{}xx-driver-libs:amd64".format(nvidia_major_version),
+        "nvidia-tesla-{}-driver-libs:amd64".format(nvidia_major_version),
+        "nvidia-alternative",
+        "nvidia-legacy-{}xx-alternative".format(nvidia_major_version),
+        "nvidia-tesla-{}-alternative".format(nvidia_major_version),
+
+        # Ubuntu 18.04+ packages
+        "libnvidia-gl-{}:amd64".format(nvidia_major_version),
+        "libnvidia-gl-{}-server:amd64".format(nvidia_major_version),
+    ):
+        nvidia_packages_to_check[name] = Package(name, [])
+
+    update_installed_packages(nvidia_packages_to_check)
+
+    for pkg_name in nvidia_packages_to_check:
+        if nvidia_packages_to_check[pkg_name].installed:
+            if pkg_name.endswith("-alternative"):
+                # If we have a Nvidia kernel module and the "*-alternative"
+                # package is installed, we want to ensure that the related
+                # user-space "*-driver-libs" are also installed, in both
+                # amd64 and i386 architectures
+                driver_libs = pkg_name.replace("-alternative", "-driver-libs")
+                nvidia_packages_to_expect[driver_libs + ":amd64"] = Package(
+                    driver_libs + ":amd64", []
+                )
+                nvidia_packages_to_expect[driver_libs + ":i386"] = Package(
+                    driver_libs + ":i386", []
+                )
+            else:
+                # Ensure that the i386 variant is installed too
+                assert pkg_name.endswith(":amd64")
+                name_i386 = pkg_name.replace(":amd64", ":i386")
+                nvidia_packages_to_expect[name_i386] = Package(name_i386, [])
+
+    if len(nvidia_packages_to_expect) == 0:
+        print("Unable to determine whether the expected Nvidia drivers "
+              "are available.")
+        print("The Steam client may have limited functionality.")
+
+    return nvidia_packages_to_expect
+
+
 def remap_package(name):
     if name in (
             'python-apt',
@@ -237,11 +327,24 @@ def remap_package(name):
         xserver = 'xserver-xorg-core-lts-' + lts
         if xserver in cache and cache[xserver].is_installed:
             if name in (
+                    'libegl1-mesa',
+                    'libgbm1',
                     'libgl1-mesa-glx',
                     'libgl1-mesa-dri',
             ):
                 return name + '-lts-' + lts
 
+            if name in (
+                    'libegl1',
+            ):
+                return name + '-mesa-lts-' + lts
+
+    if name in ('libegl1', 'libegl1-mesa'):
+        if is_glvnd():
+            return 'libegl1'
+        else:
+            return 'libegl1-mesa'
+
     if name == 'libgl1-mesa-glx':
         if is_glvnd():
             return 'libgl1'
@@ -310,7 +413,10 @@ def create_package(description):
 
 
 ###
-def get_terminal_command_line(title):
+def get_terminal(
+    title       # type: str
+):
+    # type: (...) -> typing.List[str]
     """
     Function to find a useful terminal like xterm or compatible
     """
@@ -359,91 +465,178 @@ def get_terminal_command_line(title):
                 return commandLine
 
     # Fallback if no GUI terminal program is available
-    return ['/bin/sh']
+    return ['/usr/bin/env']
+
+
+def get_terminal_wait(
+    title       # type: str
+):
+    # type: (...) -> typing.List[str]
+    return get_terminal(title) + [
+        '/bin/sh', '-euc',
+        ('e=0; "$@" || e=$?;'
+         'printf \'\\nPress return to continue: \'; read line;'
+         'exit "$e"'),
+        'sh',
+    ]
 
 
 ###
-def update_packages(packages):
+def is_root():
+    return os.getuid() == 0
+
+
+###
+def is_apt_out_of_sync():
     """
-    Function to install or update package dependencies
+    Returns True if the apt policies are not in sync with what is listed
+    in the apt sources lists.
+    """
+    # We only care about the Steam repository
+    steam_uri = 'https://repo.steampowered.com/steam'
+
+    cp = run_subprocess(
+        ['apt-cache', 'policy'],
+        capture_output=True,
+        universal_newlines=True,
+    )
+    if steam_uri in cp.stdout:
+        # The Steam repository is already in the apt policies, no need
+        # to update apt sources
+        return False
+
+    # Technically, as explained in sources.list(5), also '.sources' files
+    # with the deb822 style are understood by apt but they cannot be
+    # parsed with python-apt. By default we expect to have the Steam
+    # repository in a '.list' file, so it should be safe to skip them.
+    # However if we decide to also parse the '.sources' files, we should
+    # probably use the python3-debian package.
+    sources_list = glob.glob('/etc/apt/sources.list.d/*.list')
+    sources_list.append('/etc/apt/sources.list')
+
+    for source in sources_list:
+        try:
+            with open(source) as f:
+                for line in f.readlines():
+                    entry = SourceEntry(line, f.name)
+                    entry.parse(line)
+                    if steam_uri in entry.uri and not entry.disabled:
+                        # The Steam repository is in the sources list but not
+                        # in the apt policies. We need to update apt sources
+                        return True
+        except OSError as err:
+            print('Failed to open "{}": {}'.format(source, err))
+
+    # Apparently we don't have the Steam repository, no need to update
+    # apt sources
+    return False
+
+
+###
+def update_apt(show_progress=True):
+    logger.debug('Running apt-get update...')
+
+    if show_progress:
+        stdout_dest = subprocess.PIPE
+    else:
+        stdout_dest = subprocess.DEVNULL
+
+    proc = subprocess.Popen(
+        ['apt-get', 'update'],
+        stdout=stdout_dest,
+        bufsize=1,
+        universal_newlines=True,
+    )
+    if show_progress:
+        for _ in proc.stdout:
+            print('.', end='', flush=True)
+        print('')
+
+
+def needs_i386():
+    # type: () -> bool
+    return (get_arch() == 'amd64'
+            and 'i386' not in get_foreign_architectures())
+
+
+def enable_i386():
+    # type: () -> bool
+
+    global _foreign_architectures
+
+    if needs_i386():
+        # Check to make sure 64-bit systems can get 32-bit packages
+        logger.debug('Adding i386 architecture...')
+        cp = run_subprocess(['dpkg', '--add-architecture', 'i386'])
+        if cp.returncode != 0:
+            logger.warning(
+                'An error occurred while enabling 32-bit (i386) packages',
+            )
+            return False
+
+    # invalidate cache
+    _foreign_architectures = None
+
+    return True
+
+
+def pass_through_environ(
+    argv,       # type: typing.List[str]
+):
+    # type: (...) -> None
+
+    for var in PASS_THROUGH_ENV_VARS:
+        if var in os.environ:
+            argv.append('--setenv={}={}'.format(var, os.environ[var]))
+
+
+###
+def update_packages(packages, install_confirmation=True):
+    """
+    Function to install or update package dependencies, expected to be executed
+    in an interactive terminal.
     Ideally we would call some sort of system UI that users were familiar with
     to do this, but nothing that exists yet does what we need.
     """
 
-    package_list = " ".join([package.name for package in packages])
+    if install_confirmation:
+        print('\nSteam needs to install these additional packages:')
+        print(*packages)
+        input('\nPress return to proceed with the installation: ')
+    else:
+        logger.debug('Will install packages: %s', ' '.join(packages))
 
-    # Create a temporary file to hold the installation completion status
-    (fd, status_file) = tempfile.mkstemp()
-    os.close(fd)
+    if not is_root():
+        # We don't have root privileges, call again this script with pkexec
+        logger.debug('Re-running steamdeps as root...')
+        argv = [
+            'pkexec',
+            os.path.abspath(__file__),
+            '--interactive',
+            # We already showed the confirmation once, no need to repeat it
+            '--no-install-confirmation',
+            '--install',
+            ' '.join(packages)
+        ]
+        pass_through_environ(argv)
+        cp = run_subprocess(argv, universal_newlines=True)
+        return cp.returncode
 
-    # Create a script to run, in a secure way
-    (fd, script_file) = tempfile.mkstemp()
-    script = """#!/bin/sh
-check_sudo()
-{
-    # If your host file is misconfigured in certain circumstances this
-    # can cause sudo to block for a while, which causes gksudo to go into
-    # limbo and never return.
-    timeout --signal=9 5 sudo -v -S </dev/null 2>/dev/null
-    if [ $? -eq 124 -o $? -eq 137 ]; then
-        # sudo timed out or was killed due to timeout
-        cat <<__EOF__
-sudo timed out, your hostname may be missing from /etc/hosts.
-
-See https://support.steampowered.com/kb_article.php?ref=7493-ADXN-9620
-for more details.
-__EOF__
+    if not enable_i386():
         return 1
-    else
-        return 0
-    fi
-}
 
-cat <<__EOF__
-Steam needs to install these additional packages:
-    %s
-__EOF__
-check_sudo
+    update_apt()
 
-# Check to make sure 64-bit systems can get 32-bit packages
-if [ "$(dpkg --print-architecture)" = "amd64" ] && \
-   ! dpkg --print-foreign-architectures | grep i386 >/dev/null; then
-    sudo dpkg --add-architecture i386
-fi
+    # Install the packages using the option "--no-remove" to avoid
+    # unexpected dependencies cycle that end up removing packages that are
+    # essential for the OS to run
+    logger.debug('Installing packages: %s...', ' '.join(packages))
+    cp = run_subprocess(
+        ['apt-get', 'install', '--no-remove'] + packages,
+        universal_newlines=True,
+    )
 
-# Update the package list, showing progress
-sudo apt-get update | while read line; do echo -n "."; done
-echo
-
-# Install the packages!
-sudo apt-get install %s
-echo $? >%s
-echo -n "Press return to continue: "
-read line
-""" % (", ".join([package.name for package in packages]), package_list,
-       status_file)
-    os.write(fd, script.encode("utf-8"))
-    os.close(fd)
-    os.chmod(script_file, (stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR))
-
-    try:
-        subprocess.call(
-            get_terminal_command_line("Package Install") + [script_file])
-    except KeyboardInterrupt:
-        pass
-    os.unlink(script_file)
-
-    # Read the status out of the file, since if we ran the script in a
-    # terminal the process status will be whether the terminal started
-    try:
-        status = int(open(status_file).read())
-    except ValueError:
-        # The status wasn't written to the file
-        status = 255
-
-    os.unlink(status_file)
-
-    return status
+    return cp.returncode
 
 
 ###
@@ -480,11 +673,37 @@ def check_config(path, config):
     return True
 
 
+def update_installed_packages(packages):
+    # Get the installed package versions
+    # Make sure COLUMNS isn't set, or dpkg will truncate its output
+    if "COLUMNS" in os.environ:
+        del os.environ["COLUMNS"]
+
+    process = subprocess.Popen(['dpkg', '-l'] + list(packages.keys()),
+                               stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+    # The regex first group is the package name, the second group is the
+    # version and the third is the architecture
+    installed_pattern = re.compile(r"^\Si\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)")
+    for line in process.stdout:
+        line = line.decode("utf-8").strip()
+        match = re.match(installed_pattern, line)
+        if match is None:
+            continue
+
+        name = match.group(1)
+        if name not in packages:
+            name = name + ":" + match.group(3)
+        packages[name].set_installed(match.group(2))
+
+
 ###
 def main():
+    logging.basicConfig()
+    logging.getLogger().setLevel(logging.INFO)
+
     config = {}
     os_release = OsRelease()
-    missing_packages = []    # type: List[str]
+    missing_packages = []    # type: typing.List[str]
 
     parser = argparse.ArgumentParser(description='Install Steam dependencies')
     parser.add_argument(
@@ -497,17 +716,147 @@ def main():
         action='store_true',
         help=argparse.SUPPRESS,
     )
+    parser.add_argument(
+        '--interactive',
+        action='store_true',
+        help="This is executed in a terminal that allows user interaction",
+    )
+    parser.add_argument(
+        '--install',
+        help="Whitespace separated list of packages to install",
+    )
+    parser.add_argument(
+        '--no-install-confirmation',
+        dest='install_confirmation',
+        action='store_false',
+        help="Do not ask the user for a confirmation before attempting to "
+        "install the dependencies",
+    )
+    parser.add_argument(
+        '--setenv',
+        action='append',
+        help=argparse.SUPPRESS,
+    )
+    parser.add_argument(
+        '--update-apt',
+        action='store_true',
+        help=(
+            "Run apt-get update before evaluating the dependencies to install"
+        ),
+    )
     parser.add_argument(
         'dependencies',
         metavar='$HOME/.steam/root/steamdeps.txt',
+        nargs='?',
         help='Path to steamdeps.txt',
     )
+    parser.set_defaults(install_confirmation=True)
     args = parser.parse_args()
 
+    if args.setenv:
+        for pair in args.setenv:
+            if '=' not in pair:
+                logger.error('Expected syntax: --setenv=VAR=VALUE')
+                return 2
+
+            var, val = pair.split('=', 1)
+            os.environ[var] = val
+
+    if args.install and args.dependencies:
+        parser.print_usage(sys.stderr)
+        sys.stderr.write(
+            "The steamdeps.txt path and --install cannot both be used\n"
+        )
+        return 2
+    elif not args.install and not args.dependencies:
+        parser.print_usage(sys.stderr)
+        sys.stderr.write(
+            "One between the steamdeps.txt path and --install is required\n"
+        )
+        return 2
+
+    if 'STEAM_LAUNCHER_VERBOSE' in os.environ:
+        logging.getLogger().setLevel(logging.DEBUG)
+
+    logger.debug('Run as: %s', ' '.join([quote(a) for a in sys.argv]))
+
     if args.debug_dump_os_release:
         os_release.dump()
         return 0
 
+    if args.install:
+        if args.dry_run:
+            logger.debug('Not actually installing %r', args.install)
+            return 0
+
+        if not args.interactive:
+            argv = get_terminal_wait('Package Install') + [
+                os.path.abspath(__file__),
+                '--interactive',
+            ]
+            pass_through_environ(argv)
+
+            if not args.install_confirmation:
+                argv.append('--no-install-confirmation')
+
+            argv.extend(['--install', args.install])
+
+            logger.debug(
+                'Re-running steamdeps in a terminal to install %r...',
+                args.install,
+            )
+            cp = run_subprocess(
+                argv,
+                universal_newlines=True,
+            )
+            return cp.returncode
+
+        packages = sorted(filter(None, args.install.split(' ')))
+        return update_packages(packages, args.install_confirmation)
+
+    if args.dry_run:
+        logger.debug('Dry-run mode, avoiding side-effects')
+    elif args.update_apt or is_apt_out_of_sync() or needs_i386():
+        if not args.interactive:
+            logger.debug(
+                'Re-running steamdeps in an interactive terminal to '
+                'be able to update apt cache...',
+            )
+            argv = get_terminal_wait('Evaluating dependencies') + [
+                os.path.abspath(__file__),
+                '--interactive',
+                '--update-apt',
+            ]
+            pass_through_environ(argv)
+            argv.append(os.path.abspath(args.dependencies))
+            cp = run_subprocess(argv, universal_newlines=True)
+            return cp.returncode
+        elif not is_root():
+            # We don't have root privileges, call again this script with pkexec
+            print('The packages cache seems to be out of date')
+            input('\nPress return to update the list of available packages: ')
+
+            logger.debug(
+                'Re-running steamdeps as root to be able to update apt '
+                'cache...',
+            )
+            argv = [
+                'pkexec',
+                os.path.abspath(__file__),
+                '--interactive',
+                '--update-apt',
+            ]
+            pass_through_environ(argv)
+            argv.append(os.path.abspath(args.dependencies))
+            cp = run_subprocess(argv, universal_newlines=True)
+
+            return cp.returncode
+
+        if not enable_i386():
+            return 1
+
+        update_apt()
+
     # Make sure we can open the file
     try:
         fp = open(args.dependencies)
@@ -556,16 +905,45 @@ def main():
 
         dependencies.append(row)
 
-    if get_arch() == 'amd64':
+    ensure_installed_packages = set()       # type: typing.Set[str]
+    archs = [get_arch()]
+
+    if archs[0] == 'amd64' and 'i386' in get_foreign_architectures():
+        archs.append('i386')
+
+    for arch in archs:
         for synthetic in (
                 'libc6',
+                'libegl1',
+                'libgbm1',
                 'libgl1-mesa-dri',
                 'libgl1-mesa-glx',
         ):
-            package = create_package(synthetic + ':amd64')
+            package = create_package(synthetic + ':' + arch)
+
             if package is not None:
+                if package.name not in packages:
+                    packages[package.name] = package
+                    dependencies.append([package])
+
+                ensure_installed_packages.add(package.name)
+
+    # Try to install these packages, even if they are not
+    # listed in the steamdeps.txt file. If they are not available we
+    # just inform the user about it and continue.
+    for additional_pkg in (
+        'steam-libs-amd64:amd64',
+        'steam-libs-i386:i386',
+        'xdg-desktop-portal',
+        choose_xdg_portal_backend() or 'xdg-desktop-portal-gtk',
+    ):
+        if additional_pkg not in packages:
+            if is_package_available(additional_pkg):
+                package = Package(additional_pkg, [])
                 packages[package.name] = package
                 dependencies.append([package])
+            else:
+                missing_packages.append(additional_pkg)
 
     # The Steam container runtime (pressure-vessel) requires a setuid
     # bubblewrap executable on some kernel configurations. Steam is
@@ -584,25 +962,14 @@ def main():
         packages[package.name] = package
         dependencies.append([package])
 
-    # Try to install xdg-desktop-portal and a backend, even if they are
-    # not listed in the steamdeps.txt file. If they are not available we
-    # just inform the user about it and continue.
-    xdg_portal = 'xdg-desktop-portal'
-    if xdg_portal not in packages:
-        if is_package_available(xdg_portal):
-            package = Package(xdg_portal, [])
-            packages[package.name] = package
-            dependencies.append([package])
-        else:
-            missing_packages.append(xdg_portal)
-
-    xdg_backend = choose_xdg_portal_backend()
-    if xdg_backend is not None and xdg_backend not in packages:
-        package = Package(xdg_backend, [])
-        packages[package.name] = package
-        dependencies.append([package])
-    else:
-        missing_packages.append(xdg_backend or 'xdg-desktop-portal-gtk')
+    nvidia_packages = expected_nvidia_packages()
+    for pkg_name in nvidia_packages:
+        if pkg_name not in packages:
+            if is_package_available(pkg_name):
+                packages[pkg_name] = nvidia_packages[pkg_name]
+                dependencies.append([nvidia_packages[pkg_name]])
+            else:
+                missing_packages.append(pkg_name)
 
     if missing_packages:
         print("These packages are not available:\n")
@@ -613,32 +980,20 @@ def main():
         print("\nThe Steam client may have limited functionality.")
 
     # Print package dependency information for debug
-    """
-    for row in dependencies:
-        print " | ".join( [ str(package) for package in row ] )
-    """
+    if logger.isEnabledFor(logging.DEBUG):
+        for row in dependencies:
+            if len(row) == 0:
+                continue
 
-    # Get the installed package versions
-    # Make sure COLUMNS isn't set, or dpkg will truncate its output
-    if "COLUMNS" in os.environ:
-        del os.environ["COLUMNS"]
+            logger.debug(
+                'Dependency to be checked: %s',
+                " | ".join([str(package) for package in row]),
+            )
 
-    process = subprocess.Popen(['dpkg', '-l'] + list(packages.keys()),
-                               stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-    installed_pattern = re.compile(r"^\Si\s+([^\s]+)\s+([^\s]+)")
-    for line in process.stdout:
-        line = line.decode("utf-8").strip()
-        match = re.match(installed_pattern, line)
-        if match is None:
-            continue
-
-        name = match.group(1)
-        if name not in packages:
-            name = get_full_package_name(name)
-        packages[name].set_installed(match.group(2))
+    update_installed_packages(packages)
 
     # See which ones need to be installed
-    needed = []
+    needed = set()
     for row in dependencies:
         if len(row) == 0:
             continue
@@ -649,11 +1004,11 @@ def main():
                 satisfied = True
                 break
         if not satisfied:
-            needed.append(row[0])
+            needed.add(row[0])
 
     # If we have anything to install, do it!
     if len(needed) > 0:
-        for package in needed:
+        for package in sorted(needed, key=lambda x: x.name):
             if package.installed:
                 print("Package %s is installed with version '%s' but doesn't "
                       "match requirements: %s" % (
@@ -663,10 +1018,48 @@ def main():
                 print("Package %s needs to be installed" % package.name,
                       file=sys.stderr)
 
+        # If we are going to install additional packages, we also add the
+        # ones listed in "ensure_installed_packages". If they were already
+        # installed, this forces apt to keep them into consideration when
+        # it evaluates the new packages dependencies.
+        to_install = set([package.name for package in needed])
+        to_install.update(ensure_installed_packages)
+
         if args.dry_run:
+            print(
+                'Would run: apt-get install --no-remove {}'.format(
+                    ' '.join(sorted(to_install))
+                )
+            )
             return 1
         else:
-            return update_packages(needed)
+            if args.interactive:
+                pkgs_names = sorted(to_install)
+                return update_packages(pkgs_names, args.install_confirmation)
+
+            logger.debug(
+                'Re-running steamdeps in a terminal to install selected '
+                'packages',
+            )
+            argv = get_terminal_wait('Package Install') + [
+                os.path.abspath(__file__),
+                '--interactive',
+            ]
+            pass_through_environ(argv)
+
+            if not args.install_confirmation:
+                argv.append('--no-install-confirmation')
+
+            argv.extend([
+                '--install',
+                ' '.join(sorted(to_install)),
+            ])
+
+            cp = run_subprocess(
+                argv,
+                universal_newlines=True,
+            )
+            return cp.returncode
     else:
         return 0
 
