@@ -7,6 +7,14 @@ set -o pipefail
 shopt -s failglob
 set -u
 
+log () {
+	echo "steam.sh[$$]: $*" >&2 || :
+}
+# This version interprets backslash escapes like echo -e
+log_e () {
+	echo -e "steam.sh[$$]: $*" >&2 || :
+}
+
 # Allow us to debug what's happening in the script if necessary
 if [ "${STEAM_DEBUG-}" ]; then
 	set -x
@@ -24,7 +32,7 @@ ARCHIVE_EXT=tar.xz
 
 STEAMROOT="$(cd "$(dirname "$0")" && echo $PWD)"
 if [ -z ${STEAMROOT} ]; then
-	echo $"Couldn't find Steam root directory from "$0", aborting!"
+	log $"Couldn't find Steam root directory from "$0", aborting!"
 	exit 1
 fi
 STEAMDATA="$STEAMROOT"
@@ -33,11 +41,11 @@ if [ -z ${STEAMEXE-} ]; then
 fi
 # Backward compatibility for server operators
 if [ "$STEAMEXE" = "steamcmd" ]; then
-	echo "***************************************************"
-	echo "The recommended way to run steamcmd is: steamcmd.sh $*"
-	echo "***************************************************"
+	log "***************************************************"
+	log "The recommended way to run steamcmd is: steamcmd.sh $*"
+	log "***************************************************"
 	exec "$STEAMROOT/steamcmd.sh" "$@"
-	echo "Couldn't find steamcmd.sh" >&1
+	log "Couldn't find steamcmd.sh"
 	exit 255
 fi
 cd "$STEAMROOT"
@@ -71,20 +79,19 @@ function show_message()
 		;;
 	esac
 
-	# Show the message on standard output, for logging
-	echo -e "$title: $*"
+	log_e "$title: $*"
 
-	if [ -z "$STEAMOS" ]; then
+	if [ -n "${STEAM_ZENITY}" ]; then
 		if ! "${STEAM_ZENITY}" "$style" --text="$*" --width=450 2>/dev/null; then
 			# Save the prompt in a temporary file because it can have newlines in it
 			tmpfile="$(mktemp || echo "/tmp/steam_message.txt")"
 			echo -e "$*" >"$tmpfile"
 			xterm -bg "#383635" -fg "#d1cfcd" -T "$title" -e "cat $tmpfile; echo -n 'Press enter to continue: '; read input" 2>/dev/null || \
-				(echo "$title:"; cat "$tmpfile"; echo -n 'Press enter to continue: '; read input)
+				(echo "$title:"; cat "$tmpfile"; echo -n 'Press enter to continue: '; read input) >&2
 			rm -f "$tmpfile"
 		fi
 	else
-		# Temporary until we have a zenity equivalent for SteamOS
+		# Keep a trace on disk at least
 		echo -e "[$(date +'%Y-%m-%dT%H:%M:%S%z')]: $title: $*" >> /tmp/steam_startup_messages_$USER.txt
 	fi
 }
@@ -214,7 +221,7 @@ function install_bootstrap()
 
 	STEAMBOOTSTRAPARCHIVE=`detect_bootstrap`
 	if [ -f "$STEAMBOOTSTRAPARCHIVE" ]; then
-		echo "Installing bootstrap $STEAMBOOTSTRAPARCHIVE"
+		log "Installing bootstrap $STEAMBOOTSTRAPARCHIVE"
 		tar xf "$STEAMBOOTSTRAPARCHIVE"
 		STATUS=$?
 	else
@@ -282,7 +289,11 @@ function runtime_supported()
 
 function download_archive()
 {
-	curl -#Of "$2" 2>&1 | tr '\r' '\n' | sed 's,[^0-9]*\([0-9]*\).*,\1,' | "${STEAM_ZENITY}" --progress --auto-close --no-cancel --width 400 --text="$1\n$2"
+	if [ -n "${STEAM_ZENITY}" ]; then
+		curl -#Of "$2" 2>&1 | tr '\r' '\n' | sed 's,[^0-9]*\([0-9]*\).*,\1,' | "${STEAM_ZENITY}" --progress --auto-close --no-cancel --width 400 --text="$1\n$2"
+	else
+		curl -#Of "$2" 2>&1 | tr '\r' '\n' | sed 's,[^0-9]*\([0-9]*\).*,\1,'
+	fi
 	return ${PIPESTATUS[0]}
 }
 
@@ -300,7 +311,11 @@ function extract_archive()
 		;;
 	esac
 	if [ "${BF}" ]; then
-		tar --blocking-factor=${BF} --checkpoint=1 --checkpoint-action='exec=echo $TAR_CHECKPOINT' -xf "$2" -C "$3" | "${STEAM_ZENITY}" --progress --auto-close --no-cancel --width 400 --text="$1"
+		if [ -n "${STEAM_ZENITY}" ]; then
+			tar --blocking-factor=${BF} --checkpoint=1 --checkpoint-action='exec=echo $TAR_CHECKPOINT' -xf "$2" -C "$3" | "${STEAM_ZENITY}" --progress --auto-close --no-cancel --width 400 --text="$1"
+		else
+			tar --blocking-factor=${BF} --checkpoint=1 --checkpoint-action='exec=echo $TAR_CHECKPOINT' -xf "$2" -C "$3"
+		fi
 		return ${PIPESTATUS[0]}
 	else
 		echo "$1"
@@ -348,7 +363,7 @@ function unpack_runtime()
 	EXISTING_CHECKSUM="$(cd "$(dirname "$STEAM_RUNTIME")"; md5sum "$(basename "$STEAM_RUNTIME.$ARCHIVE_EXT")")"
 	EXPECTED_CHECKSUM="$(cat "$STEAM_RUNTIME.checksum")"
 	if [ "$EXISTING_CHECKSUM" != "$EXPECTED_CHECKSUM" ]; then
-		echo $"Runtime checksum: $EXISTING_CHECKSUM, expected $EXPECTED_CHECKSUM" >&2
+		log $"Runtime checksum: $EXISTING_CHECKSUM, expected $EXPECTED_CHECKSUM"
 		return 2
 	fi
 	if ! extract_archive $"Unpacking Steam Runtime" "$STEAM_RUNTIME.$ARCHIVE_EXT" "$EXTRACT_TMP"; then
@@ -401,7 +416,7 @@ function check_requirements()
 {
 	check_requirements_bin="$(command -v steam-runtime-check-requirements)"
 	if [ -z "$check_requirements_bin" ]; then
-		echo "Can't find 'steam-runtime-check-requirements', continuing anyway"
+		log "Can't find 'steam-runtime-check-requirements', continuing anyway"
 	else
 		message="$("$check_requirements_bin")"
 		status=$?
@@ -415,9 +430,9 @@ function check_requirements()
 			if [ -z "$message" ]; then
 				message="steam-runtime-check-requirements failed with an unexpected nonzero exit status. Trying to continue"
 			fi
-			echo $message
+			log "$message"
 		else
-			echo "Steam client's requirements are satisfied"
+			log "Steam client's requirements are satisfied"
 		fi
 	fi
 }
@@ -437,7 +452,7 @@ function reset_steam()
 
 	# Don't wipe development files
 	if [ -f "$STEAMROOT/steam_dev.cfg" ]; then
-		echo "Can't reset development directory"
+		log "Can't reset development directory"
 		return 1
 	fi
 
@@ -493,10 +508,10 @@ function reset_steam()
 
 		# Restore the steam data link
 		ln -s "$STEAMDATA" "$STEAMDATALINK"
-		echo $"Reset complete!"
+		log $"Reset complete!"
 	else
 		STATUS=1
-		echo $"Reset failed!"
+		log $"Reset failed!"
 	fi
 
 	# Okay, at this point we can recover, so re-enable interrupts
@@ -512,6 +527,9 @@ function steamos_arg()
 		if [ "$option" = "-steamos" ]; then
 			return 0; # 0 == true in bash
 		fi
+		if [ "$option" = "-steamos3" ]; then # holo/jupiter
+			return 0;
+		fi
 	done
 
 	return 1; # 1 == false in bash speak
@@ -555,6 +573,8 @@ STEAMSTARTING="$STEAMCONFIG/starting"
 : "${STEAMOS:=}"
 if steamos_arg $@; then
 	STEAMOS=1
+	# Disables progress bars and message boxes
+	STEAM_ZENITY=
 fi
 
 # See if this is the initial launch of Steam
@@ -606,7 +626,7 @@ if [ "$INITIAL_LAUNCH" ]; then
 fi
 
 # Show what we detect for distribution and release
-echo "Running Steam on $(distro_description)"
+log "Running Steam on $(distro_description)"
 
 # The Steam runtime is a complete set of libraries for running
 # Steam games, and is intended to continue to work going forward.
@@ -638,7 +658,7 @@ if [ "$STEAM_RUNTIME" = "debug" ]; then
 
 			STEAM_RUNTIME_DEBUG_ARCHIVE="$STEAM_RUNTIME_DEBUG_DIR/$(basename "$STEAM_RUNTIME_DEBUG_URL")"
 			if [ ! -f "$STEAM_RUNTIME_DEBUG_ARCHIVE" ]; then
-				echo $"Downloading debug runtime: $STEAM_RUNTIME_DEBUG_URL"
+				log $"Downloading debug runtime: $STEAM_RUNTIME_DEBUG_URL"
 				(cd "$STEAM_RUNTIME_DEBUG_DIR" && \
 					download_archive $"Downloading debug runtime..." "$STEAM_RUNTIME_DEBUG_URL")
 			fi
@@ -647,48 +667,48 @@ if [ "$STEAM_RUNTIME" = "debug" ]; then
 			fi
 		fi
 		if [ -d "$STEAM_RUNTIME_DEBUG_DIR/$STEAM_RUNTIME_DEBUG" ]; then
-			echo "STEAM_RUNTIME debug enabled, using $STEAM_RUNTIME_DEBUG"
+			log "STEAM_RUNTIME debug enabled, using $STEAM_RUNTIME_DEBUG"
 			export STEAM_RUNTIME="$STEAM_RUNTIME_DEBUG_DIR/$STEAM_RUNTIME_DEBUG"
 
 			# Set up the link to the source code
 			ln -sf "$STEAM_RUNTIME/source" /tmp/source
 		else
-			echo $"STEAM_RUNTIME couldn't download and unpack $STEAM_RUNTIME_DEBUG_URL, falling back to $STEAM_RUNTIME"
+			log $"STEAM_RUNTIME couldn't download and unpack $STEAM_RUNTIME_DEBUG_URL, falling back to $STEAM_RUNTIME"
 		fi
 	fi
 elif [ "$STEAM_RUNTIME" = "1" ]; then
-	echo "STEAM_RUNTIME is enabled by the user"
+	log "STEAM_RUNTIME is enabled by the user"
 	export STEAM_RUNTIME="$STEAMROOT/$PLATFORM/steam-runtime"
 elif [ "$STEAM_RUNTIME" = "0" ]; then
-	echo "STEAM_RUNTIME is disabled by the user"
+	log "STEAM_RUNTIME is disabled by the user"
 elif [ -z "$STEAM_RUNTIME" ]; then
 	if runtime_supported; then
-		echo "STEAM_RUNTIME is enabled automatically"
+		log "STEAM_RUNTIME is enabled automatically"
 		export STEAM_RUNTIME="$STEAMROOT/$PLATFORM/steam-runtime"
 	else
-		echo "STEAM_RUNTIME is disabled automatically"
+		log "STEAM_RUNTIME is disabled automatically"
 	fi
 else
-	echo "STEAM_RUNTIME has been set by the user to: $STEAM_RUNTIME"
+	log "STEAM_RUNTIME has been set by the user to: $STEAM_RUNTIME"
 fi
 if [ "$STEAM_RUNTIME" -a "$STEAM_RUNTIME" != "0" ]; then
 	# Unpack the runtime if necessary
 	if unpack_runtime; then
 		if [ ! -x "$STEAM_RUNTIME/setup.sh" ]; then
-			echo "internal error: $STEAM_RUNTIME/setup.sh is missing, this runtime is invalid or corrupted"
+			log "internal error: $STEAM_RUNTIME/setup.sh is missing, this runtime is invalid or corrupted"
 			exit 1
 		fi
 		export PATH="$("$STEAM_RUNTIME/setup.sh" --print-bin-path):$PATH"
 
 		if [ ! -x "$STEAM_RUNTIME/run.sh" ]; then
-			echo "internal error: $STEAM_RUNTIME/run.sh is missing, this runtime is invalid or corrupted"
+			log "internal error: $STEAM_RUNTIME/run.sh is missing, this runtime is invalid or corrupted"
 			exit 1
 		fi
 		export STEAM_RUNTIME_LIBRARY_PATH="$("$STEAM_RUNTIME/run.sh" --print-steam-runtime-library-paths)"
 
 		export LD_LIBRARY_PATH="$STEAM_RUNTIME_LIBRARY_PATH"
 	else
-		echo "Unpack runtime failed, error code $?"
+		log "Unpack runtime failed, error code $?"
 		show_message --error $"Couldn't set up the Steam Runtime. Are you running low on disk space?\nContinuing..."
 	fi
 fi
@@ -697,7 +717,7 @@ if [ "${1-}" = "--run" ]; then
 	STEAM_RUNTIME_RUN_SCRIPT="$STEAM_RUNTIME/run.sh"
 	shift
 	exec "$STEAM_RUNTIME/run.sh" "$@"
-	echo "Couldn't find run.sh" >&1
+	log "Couldn't find run.sh"
 	exit 255
 fi
 
@@ -725,6 +745,7 @@ SEGV_EXITCODE=139
 
 # and launch steam
 STEAM_DEBUGGER=${DEBUGGER-}
+: "${DEBUGGER_ARGS:=}"
 unset DEBUGGER # Don't use debugger if Steam launches itself recursively
 if [ "$STEAM_DEBUGGER" == "gdb" ] || [ "$STEAM_DEBUGGER" == "cgdb" ]; then
 	ARGSFILE=$(mktemp $USER.steam.gdb.XXXX)
@@ -763,11 +784,13 @@ elif [ "$STEAM_DEBUGGER" == "callgrind" ]; then
 	valgrind --tool=callgrind --instr-atstart=no "$STEAMROOT/$STEAMEXEPATH" "$@"
 elif [ "$STEAM_DEBUGGER" == "strace" ]; then
 	strace -osteam.strace "$STEAMROOT/$STEAMEXEPATH" "$@"
+elif [ -z "$STEAM_DEBUGGER" ]; then
+	"$STEAMROOT/$STEAMEXEPATH" "$@"
 else
 	# Most likely not what you want!
-	echo "WARNING: Using default/fallback debugger launch"
-	echo $STEAM_DEBUGGER "$STEAMROOT/$STEAMEXEPATH" "$@"
-	$STEAM_DEBUGGER "$STEAMROOT/$STEAMEXEPATH" "$@"
+	log "WARNING: Using default/fallback debugger launch"
+	echo $STEAM_DEBUGGER $DEBUGGER_ARGS "$STEAMROOT/$STEAMEXEPATH" "$@" >&2
+	$STEAM_DEBUGGER $DEBUGGER_ARGS "$STEAMROOT/$STEAMEXEPATH" "$@"
 fi
 STATUS=$?
 
@@ -777,6 +800,6 @@ export LD_LIBRARY_PATH="$SYSTEM_LD_LIBRARY_PATH"
 
 # If steam requested to restart, then restart
 if [ $STATUS -eq $MAGIC_RESTART_EXITCODE ] ; then
-	echo "Restarting Steam by request..."
+	log "Restarting Steam by request..."
 	exec "$0" "$@"
 fi
