@@ -13,9 +13,11 @@ i=0
 for script in \
     ./*.py \
     buildutils/*.py \
+    steam_launcher/*.py \
+    tests/*.py \
 ; do
     i=$((i + 1))
-    if [ "x${MYPY:="$(command -v mypy || echo false)"}" = xfalse ]; then
+    if [ "${MYPY:="$(command -v mypy || echo false)"}" = false ]; then
         echo "ok $i - $script # SKIP mypy not found"
     elif "${MYPY}" \
             --python-executable="${PYTHON:=python3}" \
