@@ -7,12 +7,14 @@
 set -e
 set -u
 
-if [ "x${PYFLAKES:=pyflakes3}" = xfalse ] || \
+if [ "${PYFLAKES:=pyflakes3}" = false ] || \
         [ -z "$(command -v "$PYFLAKES")" ]; then
     echo "1..0 # SKIP pyflakes3 not found"
 elif "${PYFLAKES}" \
     ./*.py \
     buildutils/*.py \
+    steam_launcher/*.py \
+    tests/*.py \
     >&2; then
     echo "1..1"
     echo "ok 1 - $PYFLAKES reported no issues"
